/**
 *
 */
/**
 *
 */
import GlobalMouse ;

class SimpleButton extends MovieClip {

    private static function static_symbol_name():String {

        return "__Packages.SimpleButton" ;

    }  // end private static function static_symbol_name

    private static function static_symbol_owner():Function {

        return SimpleButton ;

    }  // end private static function static_symbol_owner

    private static var static_symbol_linked:Boolean = Object.registerClass(static_symbol_name(),
                                                                            static_symbol_owner()) ;

    public static function create(parent:MovieClip,
                                    name:String,
                                    x:Number,
                                    y:Number,
                                    up_state_linkage_id:String,
                                    over_state_linkage_id:String,
                                    down_state_linkage_id:String):SimpleButton {

        var instance:SimpleButton = SimpleButton(parent.attachMovie(static_symbol_name(),
                                                                    name,
                                                                    parent.getNextHighestDepth())) ;
        instance.init(x, y, up_state_linkage_id, over_state_linkage_id, down_state_linkage_id) ;
        return instance ;

    }  // end public static function create

    private var _up_state_movie_clip:MovieClip = null ;
    private var _over_state_movie_clip:MovieClip = null ;
    private var _down_state_movie_clip:MovieClip = null ;
    private var _toggled_up_state_movie_clip:MovieClip = null ;
    private var _toggled_over_state_movie_clip:MovieClip = null ;
    private var _toggled_down_state_movie_clip:MovieClip = null ;

    // Mouse down event is triggered even if not over this button.  This flag is only enabled (true)
    // if the mouse down event occurred within this button's boundaries.
    private var _mouse_down_flag:Boolean = false ;
    private var _toggled_flag:Boolean = false ;

    public function SimpleButton() {

        GlobalMouse.static_install_global_mouse_down_listener() ;
        
    }  // end public function SimpleButton

    private function init(x:Number,
                            y:Number,
                            up_state_linkage_id:String,
                            over_state_linkage_id:String,
                            down_state_linkage_id:String) {

        if (null == over_state_linkage_id) {

            over_state_linkage_id = up_state_linkage_id ;

        }  // end if (null == over_state_linkage_id)

        if (null == down_state_linkage_id) {

            down_state_linkage_id = up_state_linkage_id ;

        }  // end if (null == down_state_linkage_id)

        _x = x ;
        _y = y ;

        _up_state_movie_clip = place_asset(up_state_linkage_id) ;
        _over_state_movie_clip = place_asset(over_state_linkage_id) ;
        _down_state_movie_clip = place_asset(down_state_linkage_id) ;

        enable_up_state() ;

    }  // end private function init

    private function place_asset(linkage_id:String):MovieClip {
        
        return attachMovie(linkage_id , _name + "_" + linkage_id, getNextHighestDepth()) ;
        
    }  // end private function place_asset

    public function is_toggle_button_enabled():Boolean {
        
        return (null != _toggled_up_state_movie_clip
                    &&
                null != _toggled_over_state_movie_clip
                    &&
                null != _toggled_down_state_movie_clip) ;
        
    }  // end public function is_toggle_button_enabled

    public function enable_toggle_button(toggled_up_state_linkage_id:String,
                                            toggled_over_state_linkage_id:String,
                                            toggled_down_state_linkage_id:String) {

        if (null == toggled_over_state_linkage_id) {

            toggled_over_state_linkage_id = toggled_up_state_linkage_id ;

        }  // end if (null == toggled_over_state_linkage_id)

        if (null == toggled_down_state_linkage_id) {

            toggled_down_state_linkage_id = toggled_up_state_linkage_id ;

        }  // end if (null == toggled_down_state_linkage_id)

        _toggled_up_state_movie_clip = place_asset(toggled_up_state_linkage_id) ;
        _toggled_over_state_movie_clip = place_asset(toggled_over_state_linkage_id) ;
        _toggled_down_state_movie_clip = place_asset(toggled_down_state_linkage_id) ;

        _toggled_up_state_movie_clip._alpha = 0 ;
        _toggled_over_state_movie_clip._alpha = 0 ;
        _toggled_down_state_movie_clip._alpha = 0 ;

    }  // end enable_toggle_button

    public function disable_toggle_button() {
        
        remove_movie_clip(_toggled_up_state_movie_clip) ;
        _toggled_up_state_movie_clip = null ;
        
        remove_movie_clip(_toggled_over_state_movie_clip) ;
        _toggled_over_state_movie_clip = null ;
        
        remove_movie_clip(_toggled_down_state_movie_clip) ;
        _toggled_down_state_movie_clip = null ;
        
    }  // end public function disable_toggle_button

    private function remove_movie_clip(movie_clip:MovieClip) {

        if (null != movie_clip)    {

            movie_clip.removeMovieClip() ;

        }  // end if (null != movie_clip)
        
    }  // end private function remove_movie_clip

    public function is_toggled():Boolean {
        
        if (is_toggle_button_enabled()) {
            
            return _toggled_flag ;
            
        }  // end if (is_toggle_button_enabled())
        
        return false ;
        
    }  // end public function is_toggled

    private function enable_up_state() {

        _over_state_movie_clip._alpha = 0 ;
        _down_state_movie_clip._alpha = 0 ;
        set_movie_clip_alpha(_toggled_over_state_movie_clip, 0) ;
        set_movie_clip_alpha(_toggled_down_state_movie_clip, 0) ;

        if (is_toggle_button_enabled() && _toggled_flag) {

            _up_state_movie_clip._alpha = 0 ;
            set_movie_clip_alpha(_toggled_up_state_movie_clip, 100) ;

        }
            
        else  // if (!is_toggle_button_enabled() || !_toggled_flag)
        {
            
            _up_state_movie_clip._alpha = 100 ;
            set_movie_clip_alpha(_toggled_up_state_movie_clip, 0) ;
            
        }  // end else if (!is_toggle_button_enabled() || !_toggled_flag)

    }  // end private function enable_up_state

    private function enable_over_state() {

        _up_state_movie_clip._alpha = 0 ;
        _down_state_movie_clip._alpha = 0 ;
        set_movie_clip_alpha(_toggled_up_state_movie_clip, 0) ;
        set_movie_clip_alpha(_toggled_down_state_movie_clip, 0) ;

        if (is_toggle_button_enabled() && _toggled_flag) {

            _over_state_movie_clip._alpha = 0 ;
            set_movie_clip_alpha(_toggled_over_state_movie_clip, 100) ;

        }
            
        else  // if (!is_toggle_button_enabled() || !_toggled_flag)
        {
            
            _over_state_movie_clip._alpha = 100 ;
            set_movie_clip_alpha(_toggled_over_state_movie_clip, 0) ;
            
        }  // end else if (!is_toggle_button_enabled() || !_toggled_flag)

    }  // end private function enable_over_state

    private function enable_down_state() {

        _up_state_movie_clip._alpha = 0 ;
        _over_state_movie_clip._alpha = 0 ;
        set_movie_clip_alpha(_toggled_up_state_movie_clip, 0) ;
        set_movie_clip_alpha(_toggled_over_state_movie_clip, 0) ;

        if (is_toggle_button_enabled() && _toggled_flag) {

            _down_state_movie_clip._alpha = 0 ;
            set_movie_clip_alpha(_toggled_down_state_movie_clip, 100) ;

        }
            
        else  // if (!is_toggle_button_enabled() || !_toggled_flag)
        {
            
            _down_state_movie_clip._alpha = 100 ;
            set_movie_clip_alpha(_toggled_down_state_movie_clip, 0) ;
            
        }  // end else if (!is_toggle_button_enabled() || !_toggled_flag)

    }  // end private function enable_down_state

    private function set_movie_clip_alpha(movie_clip:MovieClip, alpha:Number) {
        
        if (null != movie_clip) {
            
            movie_clip._alpha = alpha ;
            
        }  // end if (null != movie_clip)
        
    }  // end private function set_movie_clip_alpha

    public function onMouseDown() {
        
        if (GlobalMouse.static_is_mouse_over(this)) {

            enable_down_state() ;
            _mouse_down_flag = true ;

        }  // end if (GlobalMouse.static_is_mouse_over(this))
        
    }  // end public function onMouseDown()

    public function onMouseUp() {

        if (GlobalMouse.static_is_mouse_over(this)) {

            if (_mouse_down_flag) {
                
                _toggled_flag = !_toggled_flag ;
                
            }  // end if (_mouse_down_flag)

            enable_over_state() ;

        }  // end if (GlobalMouse.static_is_mouse_over(this))
        
        else  // if (!GlobalMouse.static_is_mouse_over(this))
        {
            
            enable_up_state() ;
            
        }  // end else if (!GlobalMouse.static_is_mouse_over(this))
        
        _mouse_down_flag = false ;        

    }  // end public function onMouseUp()

    public function onMouseMove() {

        if (GlobalMouse.static_is_mouse_over(this)) {

            if (_mouse_down_flag) {
                
                enable_down_state() ;
                
            }  // end if (_mouse_down_flag)
            
            else  // if (!_mouse_down_flag)
            {

                // If the mouse is down beginning on a different button, do not enable the over state
                // for this button.  This replicates, exactly, modern GUIs (Windows, GNOME, KDE).
                if (!GlobalMouse.static_is_mouse_down()) {

                    enable_over_state() ;
    
                }  // end if (!GlobalMouse.static_is_mouse_down())
                
            }  // end else if (!_mouse_down_flag)

        }  // end if (GlobalMouse.static_is_mouse_over(this))
        
        else  // if (!GlobalMouse.static_is_mouse_over(this))
        {
        
            enable_up_state() ;

        }  // end else if (!GlobalMouse.static_is_mouse_over(this))

    }  // end public function onMouseMove

}  // end class oas2.SimpleButton extends MovieClip
